;
; VIC20 OEM type userport joystick driver for the PET
;
; Marco van den Heuvel, 2010-01-23
;

	.include	"zeropage.inc"

	.include	"joy-kernel.inc"
	.include	"joy-error.inc"
;	.include	"pet.inc"
VIA_PRA	:= $E841		; Port register A
VIA_DDRA	:= $E843		; Data direction register A

; ------------------------------------------------------------------------
; Header. Includes jump table

.segment	"JUMPTABLE"

; Driver signature

	.byte	$6A, $6F, $79		; "joy"
	.byte	JOY_API_VERSION		; Driver API version number

; Button state masks (8 values)

	.byte	$01		; JOY_UP
	.byte	$02		; JOY_DOWN
	.byte	$04		; JOY_LEFT
	.byte	$08		; JOY_RIGHT
	.byte	$10		; JOY_FIRE
	.byte	$00		; JOY_FIRE2 unavailable
	.byte	$00		; Future expansion
	.byte	$00		; Future expansion

; Jump table.

	.addr	INSTALL
	.addr	UNINSTALL
	.addr	COUNT
	.addr	READ
	.addr	0		; IRQ entry unused

; ------------------------------------------------------------------------
; Constants

JOY_COUNT	= 1		; Number of joysticks we support


.code

; ------------------------------------------------------------------------
; INSTALL routine. Is called after the driver is loaded into memory. If
; possible, check if the hardware is present and determine the amount of
; memory available.
; Must return an JOY_ERR_xx code in a/x.
;

INSTALL:
	lda	#<JOY_ERR_OK
	ldx	#>JOY_ERR_OK
;	rts			; Run into UNINSTALL instead

; ------------------------------------------------------------------------
; UNINSTALL routine. Is called before the driver is removed from memory.
; Can do cleanup or whatever. Must not return anything.
;

UNINSTALL:
	rts


; ------------------------------------------------------------------------
; COUNT: Return the total number of available joysticks in a/x.
;

COUNT:
	lda	#<JOY_COUNT
	ldx	#>JOY_COUNT
	rts

; ------------------------------------------------------------------------
; READ: Read a particular joystick passed in A.
;


READ:
	lda	#0
	sta	VIA_DDRA
	lda	#0
	sta	tmp1
	lda	VIA_PRA
	tax
	ldy	#$80
	sty	tmp2
	ldy	#1
@joy1loop:
	txa
	and	tmp2
	beq	@notvalue
	tya
	ora	tmp1
	sta	tmp1
@notvalue:
	lda	tmp2
	lsr
	sta	tmp2
	tya
	asl
	tay
	cmp	#$20
	bne	@joy1loop
	lda	tmp1
@end1:
	eor	#$1f
	ldx	#0
	rts
